--------------------------------------------------------------------------
--                                                                      --
--                                                                      --
--               miniMIPS Processor : Register bank                     --
--                                                                      --
--                                                                      --
--                                                                      --
-- Authors : Hangouet  Samuel                                           --
--           Jan       Sbastien                                        --
--           Mouton    Louis-Marie                                      --
--           Schneider Olivier                                          --
--                                                                      --
--                                                          june 2003   --
--------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.numeric_std.all;

library work;
use work.pack_mips.all;

entity banc is
port (
       clock : in std_logic;
       reset : in std_logic;

       -- Register addresses to read
       reg_src1 : in bus5;
       reg_src2 : in bus5;

       -- Register address to write and its data
       reg_dest : in bus5;
       donnee   : in bus32;

       -- Write signal
       cmd_ecr  : in std_logic;

       -- Bank outputs
       data_src1 : out bus32;
       data_src2 : out bus32;

       wr_wait  : out std_logic
     );
end banc;


architecture rtl of banc is

    -- The register bank
    type tab_reg is array (1 to 30) of bus32;
    signal registres : tab_reg;

    signal wait_reg : bus32;  -- register 31 is wait reg

    signal adr_src1 : integer range 0 to 31;
    signal adr_src2 : integer range 0 to 31;
    signal adr_dest : integer range 0 to 31;


begin

    adr_src1 <= to_integer(unsigned(reg_src1));
    adr_src2 <= to_integer(unsigned(reg_src2));
    adr_dest <= to_integer(unsigned(reg_dest));


    data_src1 <= (others => '0') when adr_src1=0 else
                 wait_reg when adr_src1=31 else
                 registres(adr_src1);

    data_src2 <= (others => '0') when adr_src2=0 else
                 wait_reg when adr_src2=31 else
                 registres(adr_src2);

    stop_all <= '1' when cmd_ecr = '1' and adr_dest = 31 and wait_reg /= X"00000000" else
	       '0';

    process(clock)
    begin
        if clock = '1' and clock'event then
            if reset='1' then
                for i in 1 to 30 loop
                    registres(i) <= (others => '0');
                end loop;
                wait_reg <= (others =>'0');
            elsif cmd_ecr = '1' then		--write req
                if adr_dest = 31 then
                    if wait_reg = X"00000000" then
                     --  stop_all <= '0';
                        wait_reg <= donnee;
                    else
                     --   stop_all <= '1';
                        wait_reg <= wait_reg - 1;
                    end if;
                else
                    if wait_reg /= X"00000000" then
                        wait_reg <= wait_reg - 1;
                    end if;
                    if adr_dest /= 0 then
                        registres(adr_dest) <= donnee;
                    end if;
                end if;
            else -- cmd_ecr = '0'
                if wait_reg /= X"00000000" then
                    wait_reg <= wait_reg - 1;
                end if;
            end if;
        end if;
    end process;

end rtl;
