library ieee;
use ieee.std_logic_1164.all;

library work;
use work.pack_mips.all;

entity RTPRoc is
port (
    sys_clk       : in std_logic;
    sys_rst       : in std_logic;
    -- signals to video DAC
    pixel_clock   : in std_logic;
    -- Video outputs to the Video DAC
    VIDOUT_CLK     : out std_logic;
    VIDOUT_RED     : out std_logic_vector(9 downto 0);
    VIDOUT_GREEN   : out std_logic_vector(9 downto 0);
    VIDOUT_BLUE    : out std_logic_vector(9 downto 0);
    VIDOUT_BLANK_N : out std_logic;
    VIDOUT_HSYNC_N : out std_logic;
    VIDOUT_VSYNC_N : out std_logic;
    BAR_LED        : out std_logic_vector(9 downto 0)
    );

end RTPRoc;

architecture rtl of RTPRoc is
    -- Video signals to the video_controller
    signal vsync_to_video : std_logic;
    signal hsync_to_video : std_logic;
    signal blank_to_video : std_logic;
    signal load_to_video  : std_logic;
    signal byte_to_video  : std_logic_vector(7 downto 0);

    signal VS, HS, B : std_logic;

begin

   --BAR_LED(9 downto 2) <= byte_to_video;
   BAR_LED(6 downto 0) <= "1111111";

   BAR_LED(9) <= B;
   BAR_LED(8) <= HS;
   BAR_LED(7) <= VS;

VIDOUT_BLANK_N <= B;
VIDOUT_HSYNC_N <= HS;
VIDOUT_VSYNC_N <= VS;

   MM : minimips port map (
        clock => sys_clk,
        reset => sys_rst,
        vsync_to_video => vsync_to_video,
        hsync_to_video => hsync_to_video,
        blank_to_video => blank_to_video,
        load_to_video => load_to_video,
        byte_to_video => byte_to_video,

        it_mat => '0'
    );

    V_CONTROL : video_controller port map (
	Pixel_Clock => pixel_clock,
	Reset => sys_rst,

	RT_Font_Byte => byte_to_video,
	RT_Shift_Load => load_to_video,
	RT_Blank => blank_to_video,
	RT_Hsync => hsync_to_video,
	RT_VSync => vsync_to_video,

	VIDOUT_CLK  => VIDOUT_CLK,
	VIDOUT_RED => VIDOUT_RED,
	VIDOUT_GREEN=> VIDOUT_GREEN,
	VIDOUT_BLUE => VIDOUT_BLUE,
       
      	-- signals fed to LEDs for debugging    	
	VIDOUT_BLANK_N =>B,
	VIDOUT_HSYNC_N =>HS,
	VIDOUT_VSYNC_N =>VS


	--VIDOUT_BLANK_N =>VIDOUT_BLANK_N,
	--VIDOUT_HSYNC_N =>VIDOUT_HSYNC_N,
	--VIDOUT_VSYNC_N =>VIDOUT_VSYNC_N
);


end rtl;
