--------------------------------------------------------------------------------
-- Company:
-- Engineer:
--
-- Create Date:    15:00:56 05/06/05
-- Design Name:
-- Module Name:    video_controller - Behavioral
-- Project Name:
-- Target Device:
-- Tool versions:
-- Description:
--
-- Dependencies:
--
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
--
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity video_controller is
port (
        Pixel_Clock     : in std_logic;
	Reset           : in std_logic;

        --minimips output : input video controller
	RT_Font_Byte    : in std_logic_vector(7 downto 0);
	RT_Shift_Load   : in std_logic;
	RT_Blank        : in std_logic;
	RT_Hsync        : in std_logic;
	RT_VSync        : in std_logic;

        --video connections
	VIDOUT_CLK     : out std_logic;
	VIDOUT_RED     : out std_logic_vector (9 downto 0);
	VIDOUT_GREEN   : out std_logic_vector (9 downto 0);
	VIDOUT_BLUE    : out std_logic_vector (9 downto 0);
	VIDOUT_BLANK_N : out std_logic;
	VIDOUT_HSYNC_N : out std_logic;
	VIDOUT_VSYNC_N : out std_logic);
end video_controller;

architecture Behavioral of video_controller is

    -- Signals for the video controller
  signal LoadNShift : std_logic;        -- Shift register control
  signal Font_Data  : std_logic_vector(7 downto 0);   -- Input from MM to shift register
  signal ShiftData  : std_logic_vector(7 downto 0);  -- Shift register data
  signal VideoData  : std_logic;         -- Serial out ANDed with blanking

  signal HS    : std_logic:='1';
  signal VS    : std_logic:='1';
  signal Blank : std_logic:='1';

begin

    VidBlank: process (Pixel_Clock, Reset)
    begin
        if Reset = '1' then
            Blank <= '1';
        elsif Pixel_Clock'event and Pixel_Clock = '1' then
            if RT_Blank = '1' then
                Blank <= not Blank;
            end if;
        end if;
    end process VidBlank;

VidHsync: process (Pixel_Clock, Reset)
  begin
  if Reset = '1' then
      HS <= '1';
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
		if (RT_Hsync = '1') then
			HS <= not HS;
    end if;
	 end if;

  end process VidHsync;
VIDOUT_HSYNC_N <= HS;

VidVsync: process (Pixel_Clock, Reset)
  begin
  if Reset = '1' then
      VS <= '1';
  elsif Pixel_Clock'event and Pixel_Clock = '1' then
		if RT_Vsync = '1' then
			VS <= not VS;
    end if;
 end if;
  end process VidVsync;
VIDOUT_VSYNC_N <= VS;


-- Shift register
LoadNShift <= RT_Shift_Load;
Font_Data  <= RT_Font_Byte;


ShiftRegister: process (Pixel_Clock, Reset)
  begin
    if Reset = '1' then
      ShiftData <= X"AB";
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if LoadNShift = '1' then
        ShiftData <= Font_Data;
      else
        ShiftData <= ShiftData(6 downto 0) & ShiftData(7);
      end if;
    end if;
  end process ShiftRegister;

  VideoData <= ShiftData(7);

  -- Registered video signals going to the video DAC

  VideoOut: process (Pixel_Clock, Reset)
  begin
    if Reset = '1' then
    --  VIDOUT_BLANK_N <= '0';
      VIDOUT_RED   <= "0000000000";
      VIDOUT_BLUE  <= "0000000000";
      VIDOUT_GREEN <= "0000000000";
    elsif Pixel_Clock'event and Pixel_Clock = '1' then
      if VideoData = '1' then
        VIDOUT_RED   <= "1111111111";
        VIDOUT_GREEN <= "1111111111";
        VIDOUT_BLUE  <= "1111111111";
      else
        VIDOUT_RED   <= "0000000000";
        VIDOUT_GREEN <= "0000000000";
        VIDOUT_BLUE  <= "0000000000";
      end if;
    end if;

  end process VideoOut;

VIDOUT_BLANK_N <= Blank;
VIDOUT_CLK <= Pixel_Clock;

end Behavioral;
